/**
 * A class representing a standard playing card
 * 
 * @author Joel Ross
 * @version Sp14
 */
public class Card
{
	//constants for easy use of rank
	public static final int CLUBS = 1;
	public static final int DIAMONDS = 2;
	public static final int SPADES = 3;
	public static final int HEARTS = 4;
	//public enum Suit {CLUBS, DIAMONDS, SPADES, HEARTS}; //cleaner alternative; we will talk about enums later

	//constants for easy rank checking
	public static final int ACE = 1;
	public static final int JACK = 11;
	public static final int QUEEN = 12;
	public static final int KING = 13;

	//constants describing cards
	public static final int NUM_RANKS = 13;
	public static final int NUM_SUITS = 4;
	
	private int rank;
	private int suit;

	/**
	 * Constructs a new Card object with the given rank and suit
	 * @param rank an integer representing the rank (1 to 13).
	 * @param suit an integer representing the suit (1 to 4). Use CLUBS, DIAMONDS, HEARTS, SPADES as options
	 */
	public Card(int rank, int suit)
	{
		this.rank = rank;
		this.suit = suit;
	}

	/**
	 * Returns the card's rank
	 * @return the card's rank
	 */
	public int getRank()
	{
		return rank;
	}

	/**
	 * Returns the card's suit
	 * @return the card's suit
	 */
	public int getSuit()
	{
		return suit;
	}

	public String toString()
	{
		String r; //String representing rank
		if(rank == ACE)
			r = "A";
		else if(rank == JACK)
			r = "J";
		else if(rank == QUEEN)
			r = "Q";
		else if(rank == KING)
			r = "K";
		else if(rank > 0 && rank <= 10)
			r = ""+rank;
		else //bad rank
			return "Unknown Card";

		String s;
		if(suit == DIAMONDS)
			s = "D";
		else if(suit == CLUBS)
			s = "C";
		else if(suit == HEARTS)
			s = "H";
		else if(suit == SPADES)
			s = "S";
		else //bad suit
			return "Unknown Card";

		return r + " of " + s;
	}
}
