import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;


/**
 * A frame to display, shuffle, and sort a deck of cards
 * @author Joel Ross
 * @version Sp 2014
 */
public class CardFrame extends JFrame implements ActionListener
{
	private JButton shuffleButton, sortButton;
	
	private DrawnDeck deck;
	
	/**
	 * Creates a new CardFrame object
	 */
	public CardFrame()
	{
		super("Shuffle 'n' Sort");
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		this.setLayout(new BorderLayout());
		
		this.deck = new DrawnDeck();
		this.add(deck, BorderLayout.CENTER);
		
		JPanel buttonPanel = new JPanel();
		shuffleButton = new JButton("Shuffle Cards");
		shuffleButton.addActionListener(this);
		sortButton = new JButton("SortCards");
		sortButton.addActionListener(this);
		buttonPanel.add(shuffleButton);
		buttonPanel.add(sortButton);
		this.add(buttonPanel, BorderLayout.SOUTH);
		
		this.pack();
		this.setVisible(true);
	}

	//handles button events
	public void actionPerformed(ActionEvent e)
	{
		if(e.getSource() == shuffleButton)
		{
			deck.shuffle(); //shuffle
			deck.repaint(); //redraw
		}
		else if(e.getSource() == sortButton)
		{
			Thread t = new Thread(new Runnable() {  //run this in a separate thread from the main GUI thread, so that we can call repaint and pause
				public void run() { 
					deck.sort(); 
					deck.repaint(); 
				}
			});
			t.start(); //run the method contained in the above thread
		}		
	}
	
	//runs the program
	public static void main(String[] args)
	{
		new CardFrame();
	}
}
