import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;
import javax.swing.JPanel;


/**
 * A class representing a deck of cards drawn splayed on a panel
 * 
 * @author Joel Ross and YOUR NAME HERE
 * @version Sp14
 */
public class DrawnDeck extends JPanel
{
	//constants for convenience
	public static final int CARD_SPACING = 13; //space between each card
	public static final int OFFSET_W = 20; //offset on the left/right
	public static final int OFFSET_H = 50; //offset on the top/bottom
	public static final int SIZE = Card.NUM_SUITS * Card.NUM_RANKS; //number of cards in the deck

	public static final int PANEL_WIDTH = CARD_SPACING * SIZE + (DrawableCard.WIDTH - CARD_SPACING) + (OFFSET_W*2); //width of panel
	public static final int PANEL_HEIGHT = (OFFSET_H*2 + DrawableCard.HEIGHT)+DrawableCard.HEIGHT+(OFFSET_H/2); //height of panel

	public static final int WAIT_TIME_MILLIS = 100; //how long to pause between re-drawings (for animation)

	
	private DrawableCard[] deck;
	private DrawableCard[] left,right; //buffer space for sorting (instance variables for animation)

	
	/**
	 * Constructs a new standard deck of drawable cards
	 */
	public DrawnDeck()
	{
		this.setPreferredSize(new Dimension(PANEL_WIDTH,PANEL_HEIGHT));
		this.setBackground(new Color(43,105,42)); //nice green for background

		BufferedImage[][] faces = loadImageTiles("assets/cards.png", Card.NUM_RANKS, Card.NUM_SUITS);

		deck = new DrawableCard[SIZE];
		left = new DrawableCard[0];
		right = new DrawableCard[0];
		int count = 0;
		for(int i=1; i<=Card.NUM_SUITS; i++)
		{
			for(int j=1; j<=Card.NUM_RANKS; j++)
			{
				deck[count] = new DrawableCard(j, i, faces[j-1][i-1]);
				count++;
			}
		}
	}


	/**
	 * Shuffles the deck of cards--randomizes the order of the cards.
	 */
	public void shuffle()
	{
		//TODO Fill me in (and remove this comment!)
	}


	/**
	 * Sorts the deck of cards (using merge sort). The sorting is animated.
	 */
	public void sort()
	{
		//TODO Fill me in (and remove this comment!)
	}


	
	
	/**
	 * A "pause" function for use in slowing down drawing (so that you can see the algorithm). This method shows the current drawing after the pause ends.
	 * @param milliseconds The number of milliseconds to wait
	 */
	private void wait(int milliseconds)
	{
        try{
            Thread.sleep(milliseconds);
        } 
        catch (InterruptedException e){
            // ignoring exception at the moment (since we don't have multithreading that will interrupt)
        }

        repaint();
	}


	public void paintComponent(Graphics g)
	{
		super.paintComponent(g);
		Graphics2D g2d = (Graphics2D)g;

		 //draw each card in the deck
		for(int i=0; i<deck.length; i++)
		{
			if(deck[i] != null) //if not removed
				deck[i].draw(g2d, OFFSET_W+(i*CARD_SPACING), OFFSET_H);
		}


		//draw the left buffer
		for(int i=0; i<left.length; i++)
		{
			if(left[i] != null)
				left[i].draw(g2d, OFFSET_W+(i*CARD_SPACING), (int)(OFFSET_H*1.5+DrawableCard.HEIGHT));
		}

		//draw the right buffer
		for(int i=0; i<right.length; i++)
		{
			if(right[i] != null)
				right[i].draw(g2d, PANEL_WIDTH/2 - OFFSET_W + (i*CARD_SPACING), (int)(OFFSET_H*1.5+DrawableCard.HEIGHT)); //could tweak this layout to improve...
		}
	}
	

	
	/******** OLD METHDODS **********/

	/**
	 * Flips a card in the deck
	 * @param index the index of the card to flip
	 */
	public void flipCard(int index)
	{
		deck[index].flip();
		this.repaint();
	}
	
	/**
	 * Resets the deck so that all the cards are face up
	 */
	public void reset()
	{
		for(DrawableCard card : deck)
		{
			if(!card.isFaceUp())
				card.flip();
		}
		repaint();
	}
	
	/**
	 * Loads an image that provides tiles for drawing the deck of cards, and returns an array of those images
	 * Code adapted from Adam Smith.
	 * 
	 * @param filename The image file containing the tile
	 * @param numTilesWide The number of tiles wide the image is
	 * @param numTilesHigh The number of tiles high the image is
	 * @return A 2D array of BufferedImages representing the deck of cards. Array can be accessed as [rank][suit]
	 */
	public BufferedImage[][] loadImageTiles(String filename, int numTilesWide, int numTilesHigh)
	{
		BufferedImage master;

		try {
			master = ImageIO.read(new File(filename));
		}
		catch (IOException e) {
			System.err.println("Error loading " +filename+ ":");
			e.printStackTrace();
			return null;
		}

		BufferedImage[][] tiles = new BufferedImage[numTilesWide][numTilesHigh]; //initialize array
		int tileWidth = master.getWidth() / numTilesWide;
		int tileHeight = master.getHeight() / numTilesHigh;

		for (int i=0; i<tiles.length; i++)
		{
			for (int j=0; j<tiles[i].length; j++)
			{
				tiles[i][j] = master.getSubimage(i*tileWidth, j*tileHeight, tileWidth, tileHeight); //slice image
			}
		}

		return tiles;
	}
}
